SystemOrganization addCategory: #'tODE-Bibliocello'!

RBConfigurableFormatter subclass: #TOConfigurableFormatter
	instanceVariableNames: 'main renderer toView tempStack linkSelectors linkGlobals selectorAspect globalAspect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'tODE-Bibliocello'!

!TOConfigurableFormatter class methodsFor: 'class initialization' stamp: 'dkh 6/20/2011 02:43'!
initialize
	"TOConfigurableFormatter initialize"

	NumberOfArgumentsForMultiLine := 4! !

!TOConfigurableFormatter methodsFor: 'visitor-double dispatching' stamp: 'dkh 6/19/2011 09:49'!
acceptLiteralNode: aLiteralNode
	| spanClass val |
	spanClass := 'literal'.
	val := aLiteralNode value.
	(val isKindOf: Character)
		ifTrue: [ spanClass := 'character' ].
	(val isKindOf: String)
		ifTrue: [ 
			spanClass := 'string'.
			val isSymbol
				ifTrue: [ spanClass := 'symbol' ] ].
	(val isKindOf: Number)
		ifTrue: [ 
			(val isKindOf: Integer)
				ifTrue: [ spanClass := 'integer' ]
				ifFalse: [ spanClass := 'number' ] ].
	val == true
		ifTrue: [ spanClass := 'true' ].
	val == false
		ifTrue: [ spanClass := 'false' ].
	val == nil
		ifTrue: [ spanClass := 'nil' ].
	self span: [ super acceptLiteralNode: aLiteralNode ] class: spanClass! !

!TOConfigurableFormatter methodsFor: 'visitor-double dispatching' stamp: 'dkh 6/19/2011 13:23'!
acceptMethodNode: aMethodNode
	self push: aMethodNode arguments class: 'patternArg' during: [ super acceptMethodNode: aMethodNode ]! !

!TOConfigurableFormatter methodsFor: 'visitor-double dispatching' stamp: 'dkh 6/17/2011 16:35'!
acceptPragmaNode: aPragmaNode
	codeStream nextPut: $<.
	self
		formatSelectorAndArguments: aPragmaNode
		classes: #('pragmaKeyword' 'pragmaUnary' 'pragmaBinary')
		firstSeparator: [ 
			aPragmaNode selector isBinary
				ifTrue: [ self space ] ]
		restSeparator: [ self space ].
	codeStream nextPut: $>! !

!TOConfigurableFormatter methodsFor: 'visitor-double dispatching' stamp: 'dkh 6/20/2011 09:53'!
acceptVariableNode: aVariableNode
	| spanClass |
	spanClass := self spanClassForVariable: aVariableNode name.
	spanClass = 'globalVar'
		ifTrue: [ self renderGlobal: aVariableNode name label: [ self span: [ super acceptVariableNode: aVariableNode ] class: spanClass ] ]
		ifFalse: [ self span: [ super acceptVariableNode: aVariableNode ] class: spanClass ]! !

!TOConfigurableFormatter methodsFor: 'position refs' stamp: 'dkh 6/20/2011 02:46'!
appendString: aString
	codeStream appendAll: aString.
	lineStart := codeStream position! !

!TOConfigurableFormatter methodsFor: 'position refs' stamp: 'dkh 6/17/2011 16:35'!
currentLineLength
	^codeStream position - lineStart! !

!TOConfigurableFormatter methodsFor: 'private-formatting' stamp: 'dkh 6/19/2011 13:04'!
formatBlock: aBlockNode
	(LineUpBlockBrackets and: [ self willBeMultiline: aBlockNode body ])
		ifTrue: [ self newLine ]
		ifFalse: [ codeStream nextPutAll: StringInsideBlocks ].
	self
		push: aBlockNode arguments
		class: 'blockArg'
		during: [
			self formatBlockArgumentsFor: aBlockNode.
			(self willBeMultiline: aBlockNode body)
				ifTrue: [ self newLine ].
			self visitNode: aBlockNode body ].
	(LineUpBlockBrackets and: [ self willBeMultiline: aBlockNode body ])
		ifTrue: [ self newLine ]
		ifFalse: [ codeStream nextPutAll: StringInsideBlocks ]! !

!TOConfigurableFormatter methodsFor: 'private-formatting' stamp: 'dkh 6/17/2011 16:35'!
formatMethodCommentFor: aMethodNode
	originalSource isNil
		ifTrue: [ ^self ].
	self span: [ super formatMethodCommentFor: aMethodNode ] class: 'comment'! !

!TOConfigurableFormatter methodsFor: 'private-formatting' stamp: 'dkh 6/20/2011 10:08'!
formatMethodPatternFor: aMethodNode
	aMethodNode arguments isEmpty
		ifTrue: [ 
			self
				renderSelector: aMethodNode selector
				label: [ self span: [ codeStream nextPutAll: aMethodNode selector ] class: 'patternUnary' ] ]
		ifFalse: [ 
			self
				with: aMethodNode selectorParts
				and: aMethodNode arguments
				do: [ :key :arg | 
					| keyClass |
					keyClass := 'patternKeyword'.
					key value numArgs == 0
						ifTrue: [ keyClass := 'patternBinary' ].
					self
						renderSelector: aMethodNode selector
						label: [ self span: [ codeStream nextPutAll: key value ] class: keyClass ].
					self space.
					self visitNode: arg ]
				separatedBy: [ 
					MethodSignatureOnMultipleLines
						ifTrue: [ self newLine ]
						ifFalse: [ self space ] ] ]! !

!TOConfigurableFormatter methodsFor: 'private-formatting' stamp: 'dkh 6/19/2011 13:29'!
formatSelectorAndArguments: aMessageNode
	| newLineBetweenArguments |
	newLineBetweenArguments := self isMultiLineMessage: aMessageNode.
	self
		indent:
			(newLineBetweenArguments
				ifTrue: [ IndentsForKeywords ]
				ifFalse: [ 0 ])
		around: [ 
			self
				formatSelectorAndArguments: aMessageNode
classes: #('keyword' 'unary' 'binary')
				firstSeparator:
					((newLineBetweenArguments or: [ NewLineBeforeFirstKeyword ])
						ifTrue: [ [ self newLine ] ]
						ifFalse: [ [ self space ] ])
				restSeparator:
					(newLineBetweenArguments
						ifTrue: [ [ self newLine ] ]
						ifFalse: [ [ self space ] ]) ]! !

!TOConfigurableFormatter methodsFor: 'private-formatting' stamp: 'dkh 6/20/2011 10:07'!
formatSelectorAndArguments: aMessageNode classes: spanClassArray firstSeparator: firstBlock restSeparator: restBlock
	"spanClassArray: #(keyword, unary, binary)"

	| separatorBlock |
	separatorBlock := firstBlock.
	aMessageNode isUnary
		ifTrue: [ 
			(self isLineTooLong: aMessageNode selector)
				ifTrue: [ self newLine ]
				ifFalse: [ separatorBlock value ].
			self
				renderSelector: aMessageNode selector
				label: [ self span: [ codeStream nextPutAll: aMessageNode selector ] class: (spanClassArray at: 2) ] ]
		ifFalse: [ 
			aMessageNode selectorParts
				with: aMessageNode arguments
				do: [ :selector :argument | 
					(self isLineTooLong: selector value)
						ifTrue: [ self newLine ]
						ifFalse: [ separatorBlock value ].
					separatorBlock := restBlock.
					self
						indentAround: [ 
							| spanClass |
							spanClass := selector value numArgs == 0
								ifTrue: [ spanClassArray at: 3 ]
								ifFalse: [ spanClassArray at: 1 ].
							self
								renderSelector: aMessageNode selector
								label: [ self span: [ codeStream nextPutAll: selector value ] class: spanClass ].
							self
								space;
								visitNode: argument ] ] ]! !

!TOConfigurableFormatter methodsFor: 'private' stamp: 'dkh 6/19/2011 12:32'!
formatTemporariesFor: aSequenceNode
	aSequenceNode temporaries isEmpty
		ifTrue: [ ^self ].
	self
		push: aSequenceNode temporaries
		class: 'tempVar'
		during: [ 
			self
				bracketWith: '|'
				around: [ 
					self space.
					aSequenceNode temporaries
						do: [ :each | 
							self span: [ self visitNode: each ] class: 'tempVar'.
							FormatCommentWithStatements
								ifTrue: [ self formatCommentsFor: each ].
							self space ] ] ].
	self newLines: NewLinesAfterTemporaries! !

!TOConfigurableFormatter methodsFor: 'private' stamp: 'dkh 6/20/2011 13:59'!
formattedSourceFor: aNode
	^ lookaheadCode
		at: aNode
		ifAbsentPut: [ 
			| builder |
			builder := WARenderCanvas builder.
			String
				streamContents: [ :stream | 
					| context document nestedRenderer |
					document := builder documentClass on: stream codec: builder codec.
					document scriptGenerator: builder scriptGeneratorClass new.
					context := WARenderContext new.
					context document: document.
					context
						actionUrl: renderer actionUrl;
						resourceUrl: renderer context resourceUrl.
					nestedRenderer := builder rendererClass context: context.
					builder openDocument: document context: context.
					(TOConfigurableFormatter new)
						linkSelectors: self linkSelectors;
						linkGlobals: self linkGlobals;
						selectorAspect: self selectorAspect;
						globalAspect: self globalAspect;
						indent: indent;
						renderer: nestedRenderer;
						main: self main;
						tempStack: self tempStack;
						toView: toView;
						format: aNode.
					builder closeDocument: document ] ]! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 10:06'!
globalAspect
	"#lookup or #references"

	globalAspect ifNil: [ globalAspect := #lookup ].
	^ globalAspect! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 10:06'!
globalAspect: aSymbol
	"#lookup or #references"

	globalAspect := aSymbol! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 09:49'!
linkGlobals
	linkGlobals ifNil: [ linkGlobals := true ].
	^ linkGlobals! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 09:49'!
linkGlobals: aBool
	linkGlobals := aBool! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 02:20'!
linkSelectors
	linkSelectors ifNil: [ linkSelectors := true ].
	^ linkSelectors! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 02:20'!
linkSelectors: aBool
	linkSelectors := aBool! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 02:29'!
main
	^main! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 02:28'!
main: aComponent
	main := aComponent! !

!TOConfigurableFormatter methodsFor: 'position refs' stamp: 'dkh 6/17/2011 16:36'!
newLines: anInteger
	anInteger + IndentString size = 0
		ifTrue: [ codeStream space ].
	anInteger timesRepeat: [ codeStream cr ].
	lineStart := codeStream position.
	indent timesRepeat: [ codeStream nextPutAll: IndentString ]! !

!TOConfigurableFormatter methodsFor: 'private' stamp: 'dkh 6/19/2011 13:16'!
push: temporaries class: cssClass during: aBlock
	self tempStack addFirst: (temporaries collect: [ :each |each name ]) -> cssClass.
	aBlock value.
	self tempStack removeFirst! !

!TOConfigurableFormatter methodsFor: 'private-selector linking' stamp: 'dkh 6/20/2011 11:25'!
renderGlobal: globalName label: labelBlock
	self linkGlobals
		ifFalse: [ ^ labelBlock value ].
	toView
		renderLink: [ :s | 
			self globalAspect == #references
				ifTrue: [ 
					| search searchResults searchLabel |
					search := (TOSearch new)
						searchText: globalName;
						frame: toView frame.
					searchLabel := 'References to ' , globalName printString.
					search classReferencesQueryResults.
					toView pushit: (toView searchResults: searchResults labeled: searchLabel) selecting: #view main: main ]
				ifFalse: [ toView pushit: (Smalltalk at: globalName asSymbol) selecting: #view main: main ].
			self main updateCenterSideHistory: s ]
		with: labelBlock
		on: renderer! !

!TOConfigurableFormatter methodsFor: 'private-selector linking' stamp: 'dkh 6/20/2011 14:20'!
renderSelector: selector label: labelBlock
	self linkSelectors
		ifFalse: [ ^ labelBlock value ].
	toView
		renderLink: [ :s | 
			| search fr searchResults searchLabel |
			search := TOSearch new searchText: selector.
			fr := toView currentSession findFrameFor: search.
			search frame: fr.
			searchResults := self selectorAspect == #senders
				ifTrue: [ 
					searchLabel := 'Senders of ' , selector printString.
					search sendersExactQueryResults ]
				ifFalse: [ 
					searchLabel := 'Implementors of ' , selector printString.
					search implementorsExactQueryResults ].
			toView pushit: (toView searchResults: searchResults labeled: searchLabel) selecting: #view main: main.
			self main updateCenterSideHistory: s ]
		with: labelBlock
		on: renderer! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/17/2011 16:36'!
renderer: aRenderer
	renderer := aRenderer.
	codeStream := TOHtmlStream on: renderer! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 10:06'!
selectorAspect
	"#senders or #implementors"

	selectorAspect ifNil: [ selectorAspect := #senders ].
	^ selectorAspect! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/20/2011 10:06'!
selectorAspect: aSymbol
	selectorAspect := aSymbol! !

!TOConfigurableFormatter methodsFor: 'private-css' stamp: 'dkh 6/17/2011 16:36'!
span: aBlock class: spanClass
	codeStream span: aBlock class: spanClass! !

!TOConfigurableFormatter methodsFor: 'private' stamp: 'dkh 6/19/2011 13:38'!
spanClassForVariable: varName
	| spanClass |
	varName = 'super'
		ifTrue: [ ^'super' ].
	varName = 'self'
		ifTrue: [ ^'self' ].
	(toView theClass allClassVarNames includes: varName asSymbol)
		ifTrue: [ ^'classVar' ].
	(toView theClass allInstVarNames includes: varName asSymbol)
		ifTrue: [ ^'instVar' ].
	(Smalltalk includesKey: varName asSymbol)
		ifTrue: [ ^'globalVar' ].
	self tempStack
		do: [ :assoc | 
			(assoc key includes: varName)
				ifTrue: [ ^assoc value ] ].
	^'tempVar'! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/19/2011 12:30'!
tempStack
	tempStack ifNil: [ tempStack := OrderedCollection new ].
	^tempStack! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/19/2011 13:12'!
tempStack: aCollection
	tempStack := aCollection! !

!TOConfigurableFormatter methodsFor: 'accessing' stamp: 'dkh 6/17/2011 16:36'!
toView: aTOView
	toView := aTOView! !

!TOConfigurableFormatter methodsFor: 'visiting' stamp: 'dkh 6/19/2011 11:27'!
visitNode: aNode
	| needsParenthesis |
	(lookaheadCode includesKey: aNode)
		ifTrue: [^self appendString: (lookaheadCode at: aNode)].
	needsParenthesis := self needsParenthesisFor: aNode.
	self
		bracketWith:
			(needsParenthesis
				ifTrue: ['()']
				ifFalse: [''])
		around:
			[needsParenthesis ifTrue: [codeStream nextPutAll: StringInsideParentheses].
			super visitNode: aNode.
			(FormatCommentWithStatements or: [aNode isMethod or: [aNode isSequence]])
				ifFalse: [self formatCommentsFor: aNode].
			needsParenthesis
				ifTrue: [codeStream nextPutAll: StringInsideParentheses]]! !

!TOConfigurableFormatter methodsFor: 'position refs' stamp: 'dkh 6/19/2011 10:36'!
writeString: aString
	
	codeStream withLineBreaks: aString.
! !

BCMethodAnalyzer subclass: #TOMethodAnalyzer
	instanceVariableNames: 'rbParseTree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'tODE-Bibliocello'!

!TOMethodAnalyzer methodsFor: 'private' stamp: 'dkh 6/17/2011 10:52'!
parseMethodDefinition: anMCMethodDefinition 

^rbParseTree := super parseMethodDefinition: anMCMethodDefinition 
! !

!TOMethodAnalyzer methodsFor: 'accessing' stamp: 'dkh 6/17/2011 11:04'!
rbParseTree

^rbParseTree! !

Stream subclass: #TOHtmlStream
	instanceVariableNames: 'renderer position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'tODE-Bibliocello'!

!TOHtmlStream class methodsFor: 'instance creation' stamp: 'dkh 6/17/2011 16:04'!
on: aRenderer
	^ self basicNew on: aRenderer! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/20/2011 02:42'!
appendAll: aString
	renderer document nextPutAll: aString.
	position := 0! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/17/2011 16:37'!
contents
	^''! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/17/2011 16:38'!
cr
	renderer break.
	position := 0! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/17/2011 16:37'!
nextPut: aCharacter
	renderer text: (String with: aCharacter).
	position := position + 1! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/17/2011 18:02'!
nextPutAll: aString
	aString
		do: [ :char | 
			char == Character space
				ifTrue: [ renderer space ]
				ifFalse: [ 
					char == Character tab
						ifTrue: [ 
							4 timesRepeat: [ renderer space ].
							position := position + 3 ]
						ifFalse: [ renderer text: (String with: char) ] ].
			position := position + 1 ]! !

!TOHtmlStream methodsFor: 'initialization' stamp: 'dkh 6/17/2011 16:37'!
on: aRenderer
	renderer := aRenderer.
	position := 0! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/17/2011 17:03'!
position
	^position! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/17/2011 16:37'!
space
	renderer space.
	position := position + 1! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/17/2011 16:38'!
span: aBlock class: spanClass
	(renderer span) class: spanClass;
		with: [ aBlock value ]! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/20/2011 09:42'!
tab
	4 timesRepeat: [ renderer space ].
	position := position + 4! !

!TOHtmlStream methodsFor: 'accessing' stamp: 'dkh 6/20/2011 02:49'!
withLineBreaks: aString
	renderer withLineBreaks: aString.
	position := 0! !
TOConfigurableFormatter initialize!
